require "Sketchup.rb"

file_name=File.basename(__FILE__)

unless file_loaded?(file_name)
require "win32/api.rb" #win32-api version 1.4.8 http://rubygems.org/gems/win32-api/versions/
include Win32

#### Constants ####

#http://msdn.microsoft.com/en-us/library/ms632589(v=VS.85).aspx
SetWindowsHookEx=API.new('SetWindowsHookEx', 'IKII', 'I', 'User32')
UnhookWindowsHookEx=API.new('UnhookWindowsHookEx', 'I', 'I', 'User32')
CallNextHookEx=API.new('CallNextHookEx', 'IILL', 'I', 'User32')
GetCurrentThreadId=API.new('GetCurrentThreadId', 'V', 'I', 'Kernel32')
GetModuleHandle=API.new('GetModuleHandle', 'V', 'I', 'Kernel32')

class HookProc
 @@keyboardInput=nil
 @@mouseInput=nil
 @@keyboardInputLL=nil
 @@mouseInputLL=nil
 def onKeyboardInput(&block)
  @@keyboardInput=block
 end
 def onMouseInput(&block)
  @@mouseInput=block
 end
 def onLLKeyboardInput(&block)
  @@keyboardInputLL=block
 end
 def onLLMouseInput(&block)
  @@mouseInputLL=block
 end
 @@hooks={"KeyboardProc" => nil, "MouseProc" => nil, "LowLevelKeyboardProc" => nil, "LowLevelMouseProc" => nil}
 @@keyboardProc=API::Callback.new('ILI', 'I'){|nCode, wParam, lParam|
  return(CallNextHookEx.call(0, nCode, wParam, lParam)) if nCode<0
  @@keyboardInput.call(nCode, wParam, lParam).to_i
 }
 @@mouseProc=API::Callback.new('ILP', 'I'){|nCode, wParam, lParam|
  return(CallNextHookEx.call(0, nCode, wParam, lParam)) if nCode<0
  @@mouseInput.call(nCode, wParam, lParam).to_i
 }
 @@lowLevelKeyboardProc=API::Callback.new('ILP', 'I'){|nCode, wParam, lParam|
  return(CallNextHookEx.call(0, nCode, wParam, lParam)) if nCode<0
  @@keyboardInputLL.call(nCode, wParam, lParam).to_i
 }
 @@lowLevelMouseProc=API::Callback.new('ILP', 'I'){|nCode, wParam, lParam|
  return(CallNextHookEx.call(0, nCode, wParam, lParam)) if nCode<0
  @@mouseInputLL.call(nCode, wParam, lParam).to_i
 }
 def setHook(hook_id, hMod=0, threadID=0, &block)
  n=nil
  case hook_id
   when 2:
	n=@@hooks["KeyboardProc"]
	@@keyboardInput=block
	@@hooks["KeyboardProc"]=SetWindowsHookEx.call(2, @@keyboardProc, hMod, threadID)
   when 7:
	n=@@hooks["MouseProc"]
	@@mouseInput=block
	@@hooks["MouseProc"]=SetWindowsHookEx.call(7, @@mouseProc, hMod, threadID)
   when 13:
	n=@@hooks["LowLevelKeyboardProc"]
	@@keyboardInputLL=block
	@@hooks["LowLevelKeyboardProc"]=SetWindowsHookEx.call(13, @@lowLevelKeyboardProc, hMod, threadID)
   when 14:
	n=@@hooks["LowLevelMouseProc"]
	@@mouseInputLL=block
	@@hooks["LowLevelMouseProc"]=SetWindowsHookEx.call(14, @@lowLevelMouseProc, hMod, threadID)
  end
  UnhookWindowsHookEx.call(n) if n
 end
 def self.unhook(hook)
  return unless @@hooks[hook]
  UnhookWindowsHookEx.call(@@hooks[hook])
  @@hooks[hook]=nil
 end
 def self.unhookAll()
  for n in 0...@@hooks.length
   if @@hooks[@@hooks.keys[n]]
	UnhookWindowsHookEx.call(@@hooks[@@hooks.keys[n]])
	@@hooks[@@hooks.keys[n]]=nil
   end
  end
 end
end

class MyAPPObserver < Sketchup::AppObserver
 def onQuit() #Called when sketchup closes
  HookProc.unhookAll() #Call method to cleanly deactivate the hook procedure. Recommended!!!
 end
end

Sketchup.add_observer(MyAPPObserver.new)

inputProc=HookProc.new
hMod=GetModuleHandle.call
threadID=GetCurrentThreadId.call
count=0
$procState={"KeyboardProc" => 0, "MouseProc" => 0, "LowLevelKeyboardProc" => 0, "LowLevelMouseProc" => 0}
$inputText=[nil, [], nil]
items=[]

def reset()
 HookProc.unhookAll
 $inputText[0].erase! unless ($inputText[0]==nil or $inputText[0].deleted?)
 $inputText[0]=nil
 Sketchup.active_model.materials.remove($inputText[2]) unless ($inputText[2]==nil or $inputText[2].deleted?)
 $procState={"KeyboardProc" => 0, "MouseProc" => 0, "LowLevelKeyboardProc" => 0, "LowLevelMouseProc" => 0}
end

def checkToRemoveText()
 value=0
 $procState.values.each{|n|
  value+=n
 }
 reset if value==0
end

def logLine(str, lines=40, color=[0,0,0])
 if ($inputText[0]==nil or $inputText[0].deleted?)
  $inputText[0]=Sketchup.active_model.add_note("Sketchup Hook Procedure", 0.01, 0.2)
  if Sketchup.version.to_i>7 #Because there is no material.remove function
   Sketchup.active_model.materials.remove($inputText[2]) unless ($inputText[2]==nil or $inputText[2].deleted?)
   unless $inputText[0].material
	$inputText[2]=Sketchup.active_model.materials.add("MSPhysicsText")
	$inputText[0].material=$inputText[2]
   else
	$inputText[2]=$inputText[0].material
	$inputText[2].name="MSPhysicsText"
   end
   $inputText[2].color=color
  end
 end
 $inputText[1].push(str)
 if $inputText[1].length>lines then $inputText[1][0,($inputText[1].length-lines)]=nil end
 $inputText[0].text=$inputText[1].join("\n")
end

submenu=UI.menu("Plugins").add_submenu("HookProc")
items[0]=submenu.add_item("KeyboardProc"){
 if $procState["KeyboardProc"]==0
  inputProc.setHook(2, 0, threadID){|nCode, wParam, lParam|
   #script
   count+=1
   logLine("#{count} KeyboardProc: code: #{nCode} wParam: #{wParam} lParam: #{lParam}")
   reset if wParam==27 #Esc Key
   3 #0 - pass the message, 3 - remove message
   #You can also remove certain messages:
   #case wParam
   # when 10: 3 # remove
   # when 132: 0 #proccess the message
   # when 140:
   #  @rotate+=1 or something like that
   #  3 #remove the message
   #else
   # 3
   #end
  }
  $procState["KeyboardProc"]=1
 else
  HookProc.unhook("KeyboardProc")
  $procState["KeyboardProc"]=0
  checkToRemoveText
 end
}
items[1]=submenu.add_item("MouseProc"){
 if $procState["MouseProc"]==0
  inputProc.setHook(7, 0, threadID){|nCode, wParam, lParam|
   count+=1
   x=lParam[0].to_i+(lParam[1].to_i<<8)
   logLine("#{count} MouseProc: code: #{nCode} wParam: #{wParam} x: #{x}")
   0
  }
  $procState["MouseProc"]=1
 else
  HookProc.unhook("MouseProc")
  $procState["MouseProc"]=0
  checkToRemoveText
 end
}
items[2]=submenu.add_item("LowLevelKeyboardProc"){
 if $procState["LowLevelKeyboardProc"]==0
  inputProc.setHook(13, hMod, 0){|nCode, wParam, lParam|
   count+=1
   vkCode=lParam[0].to_i#+(lParam[0].to_i<<8)+(lParam[0].to_i<<16)+(lParam[0].to_i<<32)
   logLine("#{count} LowLevelKeyboardProc: code: #{nCode} wParam: #{wParam} vkCode: #{vkCode}")
   reset if vkCode==27 #Esc key http://msdn.microsoft.com/en-us/library/dd375731(v=vs.85).aspx
   3#nCode #WARNING if put 0 then keyboard messages in all windows will be removed.
  }
  $procState["LowLevelKeyboardProc"]=1
 else
  HookProc.unhook("LowLevelKeyboardProc")
  $procState["LowLevelKeyboardProc"]=0
  checkToRemoveText
 end
}
items[3]=submenu.add_item("LowLevelMouseProc"){
 if $procState["LowLevelMouseProc"]==0
  inputProc.setHook(14, hMod, 0){|nCode, wParam, lParam|
   count+=1
   logLine("#{count} LowLevelMouseProc: code: #{nCode} wParam: #{wParam} lParam: #{lParam}")
   nCode #WARNING if 0 then all mouse messages from all windows will be removed
  }
  $procState["LowLevelMouseProc"]=1
 else
  HookProc.unhook("LowLevelMouseProc")
  $procState["LowLevelMouseProc"]=0
  checkToRemoveText
 end
}
submenu.set_validation_proc(items[0]){if $procState["KeyboardProc"]==0 then MF_UNCHECKED else MF_CHECKED end}
submenu.set_validation_proc(items[1]){if $procState["MouseProc"]==0 then MF_UNCHECKED else MF_CHECKED end}
submenu.set_validation_proc(items[2]){if $procState["LowLevelKeyboardProc"]==0 then MF_UNCHECKED else MF_CHECKED end}
submenu.set_validation_proc(items[3]){if $procState["LowLevelMouseProc"]==0 then MF_UNCHECKED else MF_CHECKED end}
submenu.add_item("ResetAll"){reset}
submenu.add_item("ClearTextQueue"){
 $inputText[1]=[]
 $inputText[0].text="Cleared!" unless ($inputText[0]==nil or $inputText[0].deleted?)
}
file_loaded(file_name)
end

#Windows API Reference: http://msdn.microsoft.com/en-us/library/aa383749(VS.85).aspx
#Data Type RAnges: http://msdn.microsoft.com/en-us/library/s3f49ktz(v=VS.80).aspx
#Windows Data Types: http://msdn.microsoft.com/en-us/library/aa383751(VS.85).aspx
#System Defined Messages: http://msdn.microsoft.com/en-us/library/ms644927(v=VS.85).aspx#system_defined
#Hooks: http://msdn.microsoft.com/en-us/library/ms632589(v=VS.85).aspx